package cz.muni.fi.pb162.test3;

/**
 * Tato trida reprezentuje lokomotivu.
 *
 * @author Petr Adamek
 */
public class Locomotive extends AbstractRailVehicle implements PropulsiveVehicle {
    
    private double maximumWeight;
    private Traction traction;
    
    /**
     * Vytvori novou lokomotivu.
     *
     * @param weight hmotnost lokomotivy.
     * @param maximumSpeed maximalni rychlost
     * @param maximumWeight maximalni hmotnost tazeneho vlaku
     * @param traction trakce (tj. zpusob napajeni)
     */
    public Locomotive(double weight, double maximumSpeed, 
            double maximumWeight, Traction traction) {
        super(weight,maximumSpeed);
        if (maximumWeight < weight) {
            throw new IllegalArgumentException("maximuWeight");
        }
        if (traction == null) {
            throw new NullPointerException("traction");
        }
        this.maximumWeight = maximumWeight;
        this.traction = traction;
    }
    
    /**
     * Vrati popis lokomotivy.
     *
     * @return popis lokomotivy
     */
    public String toString() {
        return "Locomotive (weight: " + getWeight() + ", maximumWeight: " + 
                maximumWeight + ", traction: " + traction +")";
    }
    
    /**
     * Vrati druh trakce.
     *
     * @return druh trakce
     */
    public Traction getTraction() {
        return traction;
    }
    
    /**
     * Vrati maximalni hmotnost tazeneho vlaku.
     *
     * @return maximalni hmotnost tazeneho vlaku
     */
    public double getMaximumWeight() {
        return maximumWeight;
    }    
   
}
