package cz.muni.fi.pb162.test3;

/**
 * Toto rozhrani reprezentuje vlak. Vlak obsahuje alespon jedno hnaci vozidlo 
 * (lokomotivu nebo motorovy vuz) a zadny nebo vice hnanych vozidel (vagonu).
 *
 * Trida TrainImpl bude obsahovat konstruktor, ktery bude mit promenny pocet
 * parametru typu RailVehicle, ktere budou reprezentovat vozy, z nichz se vlak
 * sklada. Konstruktor musi zkontrolovat, ze vlak ma alespon jednu lokomotivu
 * nebo jine hnaci vozidlo, jinak vyhodi PropulsionMissingException.
 *
 * @author Petr Adamek
 */
public interface Train {
    
    /**
     * Vrati celkovou hmotnost vlaku. Ta se spocita jako soucet hmotnosti vsech 
     * vozidel, z nichz se vlak sklada.
     *
     * @return celkovou hmotnost vlaku
     */
    double getWeight();
    
    /**
     * Vrati maximalni rychlost vlaku v km/h. Ta je urcena maximalni rychlosti 
     * nejpomalejsiho vozidla ve vlaku (at uz lokomotivy, motoroveho vozidla,
     * nebo vagonu). Pokud neni na zacatku hnaci vozidlo, maximalni rychlost 
     * je omezena na 30 km/h.
     *
     * Aby bylo mozne vubec zjistit rychlost vlaku, vlak musi byt schopen 
     * pohybu. To znamena, ze soucet maximalni hmotnosti nakladu vsech hnacich 
     * vozidel musi byt vetsi, nez celkova hmotnost vlaku (lidove receno, 
     * lokomotivy a motorove vozy musi byt dohromady schopny utahnout vahu 
     * celeho vlaku). 
     *
     * @return maximalni rychlost vlaku
     * @throws WeakPropulsionException kdyz je vlak tezsi, nez je soucet 
     * maximalni hmotnosti nakladu vsech hnacich vozidel.
     */
    double getMaximumSpeed() throws WeakPropulsionException;
    
    /**
     * Vrati maximalni pocet pasazeru, ktery se vejde do vlaku. Ten se spocita 
     * jako soucet kapacity vsech vozu, ktere jsou schopny vezt cestujici.
     *
     * @return maximalni pocet pasazeru, ktery se vejde do vlaku
     */
    int getPassengerCapacity();
        
    /**
     * Pripoji zadane vozidlo na konec vlaku.
     *
     * @param vehicle vozidlo, ktere ma byt pripojeno na konec vlaku
     * @throws IllegalArgumentException pokud zadany vuz uz ve vlaku je
     */
    void addLast(RailVehicle vehicle);

    /**
     * Pripoji zadane vozidlo na zacatek vlaku.
     *
     * @param vehicle vozidlo, ktere ma byt pripojeno na zacatek vlaku
     * @throws IllegalArgumentException pokud zadany vuz uz ve vlaku je
     */
    void addFirst(RailVehicle vehicle);
    
    /**
     * Z vlaku se odebere zadane vozidlo. Samozrejme je mozne odebrat pouze vozidlo 
     * na zacatku nebo na konci vlaku.
     *
     * @param vehicle vozidlo, ktere chci odebrat
     * @throws IllegalArgumentException pokud zadany vuz ve vlaku neni
     * @throws cz.muni.fi.pb162.test3.TrainException pokud se pokousim odebrat 
     * vozidlo, ktere neni na zacatku nebo na konci vlaku
     * @throws cz.muni.fi.pb162.test3.PropulsionMissingException pokud se pokousim 
     * odebrat posledni hnaci vozidlo
     */
    void removeVehicle(RailVehicle vehicle) throws PropulsionMissingException, 
            TrainException;    
    
    /**
     * Vypise popis vlaku ve formatu:
     * 
     * Vlak: [&lt;prvni vuz&gt;, &lt;druhy vuz&gt;, ...].
     * @return popis vlaku ve vyse uvedenem formatu.
     */
    String toString();
}
