package cz.muni.fi.pb162.test3;

import junit.framework.TestCase;

public class TrainTest extends TestCase {
    
    public TrainTest(String testName) {
        super(testName);
    }

    Locomotive l1 = new Locomotive(100000,160,300000,Traction.ELECTRIC);
    Locomotive l2 = new Locomotive(50000,40,200000,Traction.ELECTRIC);
    Locomotive l3 = new Locomotive(100000,160,5500000,Traction.ELECTRIC);
    MultipleUnit m1 = new MultipleUnit(40000,80,120000,Traction.ELECTRIC,80);
    SleepingCar sc1 = new SleepingCar(20000,120,30);
    Wagon w1 = new Wagon(15000,30);
    Wagon w2 = new Wagon(16000,45);
    Wagon veryheavy = new Wagon(700000,5);
    
    public void testNewTrain() throws PropulsionMissingException {
    
        try {
            Train t1 = new TrainImpl();
            fail("Ocekava se PropulsionMissingException ");
        } catch(PropulsionMissingException ex) {
        }
        try {
            Train t1 = new TrainImpl(w1);
            fail("Ocekava se PropulsionMissingException ");
        } catch(PropulsionMissingException ex) {
        }
        try {
            Train t1 = new TrainImpl(w1,sc1);
            fail("Ocekava se PropulsionMissingException ");
        } catch(PropulsionMissingException ex) {
        }

        try {
            Train t1 = new TrainImpl(w1,sc1,w2);
            fail("Ocekava se PropulsionMissingException ");
        } catch(PropulsionMissingException ex) {
        }

        // Tohle vsechno by melo projit
        new TrainImpl(l1);
        new TrainImpl(l2);
        new TrainImpl(m1);
        new TrainImpl(m1,sc1);
        new TrainImpl(l1,w2);
        new TrainImpl(l1,w2,m1,sc1,l2);
        new TrainImpl(l1,w2,m1,sc1,veryheavy);

        new TrainImpl(new PropulsiveVehicle() {
            public double getMaximumSpeed() {
                throw new UnsupportedOperationException();
            }
            public double getMaximumWeight() {
                throw new UnsupportedOperationException();
            }
            public Traction getTraction() {
                throw new UnsupportedOperationException();
            }
            public double getWeight() {
                throw new UnsupportedOperationException();
            }
        });
    }

    public void testGetWeght() throws PropulsionMissingException {
        assertEquals(100000.0, new TrainImpl(l1).getWeight());
        assertEquals(50000.0, new TrainImpl(l2).getWeight());
        assertEquals(40000.0, new TrainImpl(m1).getWeight());
        assertEquals(60000.0, new TrainImpl(m1,sc1).getWeight());
        assertEquals(116000.0, new TrainImpl(l1,w2).getWeight());
        assertEquals(226000.0, new TrainImpl(l1,w2,m1,sc1,l2).getWeight());
        assertEquals(876000.0, new TrainImpl(l1,w2,m1,sc1,veryheavy).getWeight());
    }

    public void testGetPassengerCapacity() throws PropulsionMissingException {
        assertEquals(0, new TrainImpl(l1).getPassengerCapacity());
        assertEquals(0, new TrainImpl(l2).getPassengerCapacity());
        assertEquals(80, new TrainImpl(m1).getPassengerCapacity());
        assertEquals(110, new TrainImpl(m1,sc1).getPassengerCapacity());
        assertEquals(0, new TrainImpl(l1,w2).getPassengerCapacity());
        assertEquals(110, new TrainImpl(l1,w2,m1,sc1,l2).getPassengerCapacity());
        assertEquals(110, new TrainImpl(l1,w2,m1,sc1,veryheavy).getPassengerCapacity());
        assertEquals(115, new TrainImpl(m1,sc1,new PassengerCar() {
            public double getMaximumSpeed() {
                throw new UnsupportedOperationException();
            }
            public int getPassengerCapacity() {
                return 5;
            }
            public double getWeight() {
                throw new UnsupportedOperationException();
            }
        }).getPassengerCapacity());
    };

    public void testToString() throws PropulsionMissingException {
        assertEquals("Vlak: [MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80)]", new TrainImpl(m1).toString());
        assertEquals("Vlak: [MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80), " +
                "SleepingCar (weight: 20000.0, passengerCapacity: 30)]", new TrainImpl(m1,sc1).toString());
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC), Wagon (weight: 16000.0)]", new TrainImpl(l1,w2).toString());
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC), Wagon (weight: 16000.0), " +
                "MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80), " +
                "SleepingCar (weight: 20000.0, passengerCapacity: 30), " +
                "Locomotive (weight: 50000.0, maximumWeight: 200000.0, traction: ELECTRIC)]", new TrainImpl(l1,w2,m1,sc1,l2).toString());
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC), Wagon (weight: 16000.0), " +
                "MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80), " +
                "SleepingCar (weight: 20000.0, passengerCapacity: 30), Wagon (weight: 700000.0)]", new TrainImpl(l1,w2,m1,sc1,veryheavy).toString());
    }

    public void testAddLast() throws PropulsionMissingException {
        Train t = new TrainImpl(l1);
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC)]",t.toString());
        t.addLast(m1);
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC), " +
                "MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80)]",t.toString());
        try {
            t.addLast(m1);
            fail("Ocekava se vyjimka IllegalArgumentException");
        } catch(IllegalArgumentException ex) {
        }
    }

    public void testAddFirst() throws PropulsionMissingException {
        Train t = new TrainImpl(l1);
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC)]",t.toString());
        t.addFirst(m1);
        assertEquals("Vlak: [MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80), " +
                "Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC)]",t.toString());
        try {
            t.addFirst(m1);
            fail("Ocekava se vyjimka IllegalArgumentException");
        } catch(IllegalArgumentException ex) {
        }
    }

    public void testRemoveVehicle() throws PropulsionMissingException, TrainException {
        Train t = new TrainImpl(m1,l1);
        assertEquals("Vlak: [MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80), " +
                "Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC)]",t.toString());
        t.removeVehicle(m1);
        assertEquals("Vlak: [Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC)]",t.toString());

        t = new TrainImpl(m1,l1);
        assertEquals("Vlak: [MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80), " +
                "Locomotive (weight: 100000.0, maximumWeight: 300000.0, traction: ELECTRIC)]",t.toString());
        t.removeVehicle(l1);
        assertEquals("Vlak: [MultipleUnit (weight: 40000.0, maximumWeight: 120000.0, traction: ELECTRIC, passengerCapacity: 80)]",t.toString());
        
        t = new TrainImpl(m1,l1);
        try {
            t.removeVehicle(w1);
            fail("Ocekava se vyjimka IllegalArgumentException");
        } catch(IllegalArgumentException ex) {
        }
        
        t = new TrainImpl(m1,w1,l1);
        try {
            t.removeVehicle(w1);
            fail("Ocekava se vyjimka TrainException");
        } catch(TrainException ex) {
        }

        t = new TrainImpl(m1,w1,l1);
        t.removeVehicle(m1);
        try {
            t.removeVehicle(l1);
            fail("Ocekava se vyjimka PropulsionMissingException ");
        } catch(PropulsionMissingException  ex) {
        }

        t = new TrainImpl(m1,w1,l1);
        t.removeVehicle(l1);
        try {
            t.removeVehicle(m1);
            fail("Ocekava se vyjimka PropulsionMissingException ");
        } catch(PropulsionMissingException  ex) {
        }
    }

    public void testgetMaximumSpeed() throws PropulsionMissingException, 
            WeakPropulsionException {
        
        assertEquals(160.0, new TrainImpl(l1).getMaximumSpeed());
        assertEquals(40.0, new TrainImpl(l2).getMaximumSpeed());
        assertEquals(80.0, new TrainImpl(m1).getMaximumSpeed());
        assertEquals(80.0, new TrainImpl(m1,sc1).getMaximumSpeed());
        assertEquals(45.0, new TrainImpl(l1,w2).getMaximumSpeed());
        assertEquals(40.0, new TrainImpl(l1,w2,m1,sc1,l2).getMaximumSpeed());
        assertEquals(5.0, new TrainImpl(l1,l2,l3,veryheavy).getMaximumSpeed());

        // Vlaky, co na zacatku nemaji lokomotivu
        assertEquals(5.0, new TrainImpl(veryheavy,l1,l2,l3).getMaximumSpeed());
        assertEquals(30.0, new TrainImpl(w2,l1,m1,sc1,l2).getMaximumSpeed());
        
        try {
            new TrainImpl(l1,w2,m1,sc1,veryheavy).getMaximumSpeed();
            fail("Ocekava se WeakPropulsionException ");
        } catch(WeakPropulsionException ex) {
        }
    }
}
